package org.apache.bcel.generic;

import java.io.DataOutputStream;
import java.io.IOException;
import org.apache.bcel.ExceptionConst;
import org.apache.bcel.ExceptionConst.EXCS;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.util.ByteSequence;

public final class INVOKEINTERFACE extends InvokeInstruction {
   private int nargs;

   INVOKEINTERFACE() {
   }

   public INVOKEINTERFACE(int index, int nargs) {
      super((short)185, index);
      super.setLength(5);
      if (nargs < 1) {
         throw new ClassGenException("Number of arguments must be > 0 " + nargs);
      } else {
         this.nargs = nargs;
      }
   }

   public void accept(Visitor v) {
      v.visitExceptionThrower(this);
      v.visitTypedInstruction(this);
      v.visitStackConsumer(this);
      v.visitStackProducer(this);
      v.visitLoadClass(this);
      v.visitCPInstruction(this);
      v.visitFieldOrMethod(this);
      v.visitInvokeInstruction(this);
      v.visitINVOKEINTERFACE(this);
   }

   public int consumeStack(ConstantPoolGen cpg) {
      return this.nargs;
   }

   public void dump(DataOutputStream out) throws IOException {
      out.writeByte(super.getOpcode());
      out.writeShort(super.getIndex());
      out.writeByte(this.nargs);
      out.writeByte(0);
   }

   public int getCount() {
      return this.nargs;
   }

   public Class<?>[] getExceptions() {
      return ExceptionConst.createExceptions(EXCS.EXCS_INTERFACE_METHOD_RESOLUTION, new Class[]{ExceptionConst.UNSATISFIED_LINK_ERROR, ExceptionConst.ABSTRACT_METHOD_ERROR, ExceptionConst.ILLEGAL_ACCESS_ERROR, ExceptionConst.INCOMPATIBLE_CLASS_CHANGE_ERROR});
   }

   protected void initFromFile(ByteSequence bytes, boolean wide) throws IOException {
      super.initFromFile(bytes, wide);
      super.setLength(5);
      this.nargs = bytes.readUnsignedByte();
      bytes.readByte();
   }

   public String toString(ConstantPool cp) {
      return super.toString(cp) + " " + this.nargs;
   }
}
